

--------------------------------------------------------------------
--                                                                           
--  txt2pdf.ex  Text to PDF converter
--                                                                           
--  Written:    1999/08/27 Maurizio Moroni
--  Revised:    2000/06/04
--                                                                            
--------------------------------------------------------------------

without warning

include file.e

  
atom page_width 
atom page_depth
atom margin
atom font_size
atom lead_size
atom object_id
atom page_tree_id
sequence PageList


page_width = 594.0
page_depth = 828.0
margin = 30.0
font_size = 8.0
lead_size = 10.0
object_id = 1
PageList = {}

atom num_pages
num_pages = 0

procedure store_page( atom id )
-- store page ids
   
   PageList = PageList & id
   num_pages += 1

end procedure

sequence xrefs
xrefs = {}

procedure start_object( atom id, integer outfile )
-- Start the definition of an object and stores cross references

   xrefs = xrefs & where( outfile )
   printf(outfile, "%d 0 obj\n", id )
   
end procedure

atom stream_id
atom stream_len_id
atom stream_start
atom ypos

procedure start_page( integer outfile )
   
   stream_id = object_id
   object_id += 1
   stream_len_id = object_id
   object_id += 1
   
   start_object( stream_id, outfile )
   printf(outfile, "<< /Length %d 0 R >>\n", stream_len_id )
   puts(outfile, "stream\n" )
   stream_start = where( outfile )
   printf(outfile, "BT\n/F0 %g Tf\n", font_size )
   ypos = page_depth - margin
   printf(outfile, "%g ", margin )
   printf(outfile, "%g Td\n", ypos )
   printf(outfile, "%g TL\n", lead_size )
   
end procedure

procedure end_page( integer outfile )

  atom stream_len
  atom page_id 
  
  page_id = object_id
  object_id += 1
  
  store_page( page_id )
  puts(outfile, "ET\n" )
  stream_len = where( outfile ) - stream_start
  puts(outfile, "endstream\nendobj\n" )
  start_object( stream_len_id, outfile )
  printf(outfile, "%d\nendobj\n", stream_len )
  start_object( page_id, outfile )
  printf(outfile, "<</Type/Page/Parent %d 0 R ", page_tree_id )
  printf(outfile, "/Contents %d 0 R>>\nendobj\n", stream_id )
  
end procedure

procedure do_text( integer infile, integer outfile )

  sequence buffer
  object line
  atom c
  integer fn
  
  buffer = {}
  
  start_page( outfile )
  
  
  while 1 do    
      
     line = gets( infile )
     if atom(line) then
	 
	exit -- no more lines, end of file, 
	  
     end if
  
     if ypos < margin then
	end_page( outfile )
	start_page( outfile )
     end if
     
     if length( line ) = 0 then
	puts(outfile, "T*\n")
     else
	
	puts(outfile, "(")
	for i = 1 to length( line ) - 1 do
	    c = line[ i ]
	       
	    if c = '(' or c = ')' or c = '\\' then
		puts(outfile, '\\' )
	    end if
	       
	    puts(outfile, c)
	       
	end for
	puts(outfile, ")'\n")
	   
    end if

    ypos = ypos - lead_size

  end while
  
  end_page( outfile )

  
  
end procedure


procedure txt2pdf( sequence infile, sequence outfile )

    atom catalog_id
    atom font_id
    atom start_xref
    
    integer fd_in
    integer fd_out
    
    fd_in = open(infile, "r")
      if fd_in = -1 then
        if length( infile ) =  0 then
           puts(1, "txt2pdf E: You must enter the source file name\n")
        else
           puts(1,"txt2pdf E: Can't open " & infile & "\n")
        end if
        return
      end if
      
    fd_out = open(outfile, "w")
    if fd_out = -1 then
        if length( outfile ) =  0 then
              puts(1, "txt2pdf E: You must enter the destination file name\n")
           else
              puts(1,"txt2pdf E: Can't open " & outfile & "\n")
        end if
        return
    end if
    
    puts(1, "\n\ntxt2pdf I: Processing " & infile & "....\n\n")
    puts(fd_out, "%%PDF-1.0\n")
    page_tree_id = object_id
    object_id += 1
    
    do_text( fd_in, fd_out)
    
    font_id = object_id
    object_id += 1
    
    start_object( font_id, fd_out )
    puts(fd_out, "<</Type/Font/Subtype/Type1/BaseFont/Courier/Encoding/WinAnsiEncoding>>\nendobj\n")
    
    start_object(page_tree_id, fd_out) 
    printf(fd_out, "<</Type /Pages /Count %d\n", num_pages)
    
    puts(fd_out, "/Kids[\n" )
    
    for i = 1 to length( PageList ) do
      printf(fd_out, "%d 0 R\n", PageList[ i ] )
    end for
    
    puts(fd_out, "]\n" )
    
    printf(fd_out, "/Resources<</ProcSet[/PDF/Text]/Font<</F0 %d 0 R>> >>\n", font_id)
    printf(fd_out, "/MediaBox [ 0 0 %g ", page_width)
    printf(fd_out, "%g ]\n", page_depth)
    puts(fd_out, ">>\nendobj\n")
    
    catalog_id = object_id
    object_id += 1
    start_object(catalog_id, fd_out)
    printf(fd_out, "<</Type/Catalog/Pages %d 0 R>>\nendobj\n", page_tree_id)
    
    start_xref = where( fd_out ) 
    puts(fd_out, "xref\n" )
    printf(fd_out, "0 %d\n", object_id)
    puts(fd_out, "0000000000 65535 f \n")
    
    for i = 1 to object_id - 1 do 
       printf(fd_out, "%010d 00000 n \n", xrefs[i] )
    end for
    
    printf(fd_out, "trailer\n<<\n/Size %d\n", object_id )
    printf(fd_out, "/Root %d 0 R\n>>\n", catalog_id )
    printf(fd_out, "startxref\n%d\n%%%%EOF\n", start_xref)
    
    close( fd_out )
    close( fd_in )
    
    
end procedure

sequence cmd

puts(1, "txt2pdf - A text to PDF file converter V1.0 (alpha)\n")
puts(1, "Written by Maurizio Moroni 1999-2001\n")

cmd = command_line()

if length(cmd) != 4 then
   puts(1, "\nsyntax: txt2pdf <file to convert> <resulting PDF file>\n\n")
else 
   txt2pdf( cmd[3], cmd[4])
end if



